<?php declare(strict_types=1);

namespace Omnikassa\Payment\Helpers;

use Shopware\Core\Checkout\Payment\PaymentMethodEntity;

use Shopware\Core\Framework\Context;
use Shopware\Core\Framework\DataAbstractionLayer\EntityRepositoryInterface;
use Shopware\Core\Framework\DataAbstractionLayer\Exception\InconsistentCriteriaIdsException;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Criteria;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Filter\EqualsFilter;
use Shopware\Core\System\SystemConfig\SystemConfigService;

use Omnikassa\Payment\Components\OmnikassaPayment\OmnikassaService;

use Symfony\Component\Routing\Generator\UrlGeneratorInterface;

class OmnikassaHelper
{
    /**
     * @var SystemConfigService
     */
    public $oSystemConfigService;

    /**
     * @var OmnikassaService
     */
    private $oOmnikassaClient;

    /** @var UrlGeneratorInterface $oRouter */
    private $oRouter;

    /** @var EntityRepositoryInterface */
    private $oPaymentRepository;

    public function __construct(
		SystemConfigService $oSystemConfigService,
		OmnikassaService $oOmnikassaClient,
		UrlGeneratorInterface $oRouter,
		EntityRepositoryInterface $oPaymentRepository) {

        $this->systemConfigService = $oSystemConfigService;
        $this->omnikassaClient = $oOmnikassaClient;
        $this->router = $oRouter;
        $this->paymentRepository = $oPaymentRepository;
    }

    public function createOmnikassaClient(?string $sSalesChannelId): OmnikassaService {
        if ($this->omnikassaClient === null) {
            $this->initOmnikassaClient($sSalesChannelId);
        }

        return $this->omnikassaClient;
    }

    public function initOmnikassaClient(?string $sSalesChannelId): OmnikassaService {
        $bSandbox = $this->getSetting('sandbox', $sSalesChannelId);
		$bTestmode = false;

		if($bSandbox)
		{
			$sRefreshToken = $this->getSetting('testRefreshtoken', $sSalesChannelId);
			$sSigningKey = $this->getSetting('testSigningkey', $sSalesChannelId);
			$bTestmode = true;
		}
		else
		{
			$sRefreshToken = $this->getSetting('prodRefreshtoken', $sSalesChannelId);
			$sSigningKey = $this->getSetting('prodSigningkey', $sSalesChannelId);
			$bTestmode = false;
		}

        $this->omnikassaClient = new OmnikassaService();
        $this->omnikassaClient->setRefreshToken($sRefreshToken);
        $this->omnikassaClient->setSigningKey($sSigningKey);
        $this->omnikassaClient->setTestmode($bTestmode);

        return $this->omnikassaClient;
    }

    public function getSetting(string $sSettingKey, ?string $sSalesChannelId = null)
    {
        return $this->systemConfigService->get('OmnikassaPayment.config.' . $sSettingKey, $sSalesChannelId);
    }


    public function getYears()
    {
        $years = [];
        for($i = date("Y") - 17; $i > date("Y") - 125; $i--)
        {
            $years[] = [
                'value' => $i,
                'year' => $i
            ];
        }
        return $years;
    }

	public function mb_splitAddress($sAddress)
	{
		// Get everything up to the first number with a regex
		$bHasMatch = preg_match('/^[^0-9]*/', $sAddress, $aMatch);

		// If no matching is possible, return the supplied string as the street
		if(!$bHasMatch)
		{
			return array($sAddress, '', '');
		}

		// Remove the street from the sAddress.
		$sAddress = str_replace($aMatch[0], '', $sAddress);
		$sStreetname = trim($aMatch[0]);

		// Nothing left to split, return only the streetname
		if(strlen($sAddress) < 1)
		{
			return array($sAddress, '', '');
		}

		// Explode sAddress to an array using a multiple explode function
		$aAddress = $this->mb_multiExplodeArray(array(' ', '-', '|', '&', '/', '_', '\\'), $sAddress);

		// Shift the first element off the array, that is the house number
		$iHousenumber = array_shift($aAddress);

		// If the array is empty now, there is no extension.
		if(count($aAddress) > 0)
		{
			// Join together the remaining pieces as the extension.
			$sExtension = substr(implode(' ', $aAddress), 0, 4);
		}
		else
		{
			$sExtension = '';
		}

		return array($sStreetname, $iHousenumber, $sExtension);
	}

	private function mb_multiExplodeArray($aDelimiter, $sString)
	{
		$sInput = str_replace($aDelimiter, ' ', $sString);
		$aArray = explode(' ', $sInput);

		return $aArray;
	}


    /**
     * @return array
     */
    public function getNotifyUrl(): string
    {
        return $this->router->generate(
            'omnikassa_notify',
             [],
            UrlGeneratorInterface::ABSOLUTE_URL
        );
    }

    /**
     * Get payment method by ID.
     *
     * @param $id
     * @return PaymentMethodEntity
     * @throws InconsistentCriteriaIdsException
     */
    public function getPaymentMethodById($id) : ?PaymentMethodEntity
    {
        // Fetch ID for update
        $paymentCriteria = new Criteria();
        $paymentCriteria->addFilter(new EqualsFilter('id', $id));

        // Get payment methods
        $paymentMethods = $this->paymentRepository->search($paymentCriteria, Context::createDefaultContext());

        if ($paymentMethods->getTotal() === 0) {
            return null;
        }

        return $paymentMethods->first();
    }
}