<?php declare(strict_types=1);

namespace Omnikassa\Payment\Installers;

use Shopware\Core\Content\Media\File\FileSaver;
use Shopware\Core\Content\Media\File\MediaFile;
use Shopware\Core\Framework\Context;
use Shopware\Core\Framework\Plugin;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Criteria;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Filter\EqualsFilter;
use Shopware\Core\Framework\Plugin\Context\ActivateContext;
use Shopware\Core\Framework\Plugin\Context\DeactivateContext;
use Shopware\Core\Framework\Plugin\Context\InstallContext;
use Shopware\Core\Framework\Plugin\Context\UninstallContext;
use Shopware\Core\Framework\Plugin\Context\UpdateContext;

use Shopware\Core\Framework\Uuid\Uuid;
use Symfony\Component\Config\FileLocator;
use Symfony\Component\DependencyInjection\ContainerBuilder;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\DependencyInjection\Loader\XmlFileLoader;


use Shopware\Core\Framework\DataAbstractionLayer\EntityRepository;
use Shopware\Core\Framework\DataAbstractionLayer\EntityRepositoryInterface;
use Shopware\Core\Framework\Plugin\Util\PluginIdProvider;

use Omnikassa\Payment\Code1Rok2;
use Omnikassa\Payment\PaymentMethods\PaymentMethodInterface;
use Omnikassa\Payment\PaymentMethods\AfterpayMethod;
use Omnikassa\Payment\PaymentMethods\BancontactMethod;
use Omnikassa\Payment\PaymentMethods\CreditcardMethod;
use Omnikassa\Payment\PaymentMethods\IdealMethod;
use Omnikassa\Payment\PaymentMethods\MaestroMethod;
use Omnikassa\Payment\PaymentMethods\MastercardMethod;
use Omnikassa\Payment\PaymentMethods\PaypalMethod;
use Omnikassa\Payment\PaymentMethods\VisaMethod;
use Omnikassa\Payment\PaymentMethods\VpayMethod;

class PaymentMethodInstaller implements InstallerInterface
{
    /** @var PluginIdProvider */
    public $pluginIdProvider;
    /** @var EntityRepositoryInterface */
    public $paymentMethodRepository;

    /** @var EntityRepositoryInterface */
    private $mediaRepository;

    /** @var FileSaver */
    private $fileSaver;

    /**
     * PaymentMethodsInstaller constructor.
     * @param ContainerInterface $container
     */
    public function __construct(ContainerInterface $container)
    {
        $this->pluginIdProvider = $container->get(PluginIdProvider::class);
        $this->paymentMethodRepository = $container->get('payment_method.repository');
        $this->mediaRepository = $container->get('media.repository');
        $this->fileSaver = $container->get(FileSaver::class);
    }

    /**
     * @param InstallContext $context
     */
    public function install(InstallContext $context): void
    {
        foreach ($this->getPaymentMethods() as $paymentMethod) {
            $this->installPaymentMethod($context->getContext(), new $paymentMethod(), false);
            $this->addMedia($context->getContext(), new $paymentMethod());
        }
    }

    /**
     * @param UpdateContext $context
     */
    public function update(UpdateContext $context): void
    {
        foreach ($this->getPaymentMethods() as $paymentMethod) {
            $this->installPaymentMethod($context->getContext(), new $paymentMethod(), $context->getPlugin()->isActive());
            $this->addMedia($context->getContext(), new $paymentMethod());
        }
    }

    /**
     * @param UninstallContext $context
     */
    public function uninstall(UninstallContext $context): void
    {
        foreach ($this->getPaymentMethods() as $paymentMethod) {
            $this->setPaymentMethodActive($context->getContext(), new $paymentMethod(), false);
        }
    }

    /**
     * @param ActivateContext $context
     */
    public function activate(ActivateContext $context): void
    {
        foreach ($this->getPaymentMethods() as $paymentMethod) {
            $this->setPaymentMethodActive($context->getContext(), new $paymentMethod(), true);
        }
    }

    /**
     * @param DeactivateContext $context
     */
    public function deactivate(DeactivateContext $context): void
    {
        foreach ($this->getPaymentMethods() as $paymentMethod) {
            $this->setPaymentMethodActive($context->getContext(), new $paymentMethod(), false);
        }
    }

    /**
     * @param Context $context
     * @param PaymentMethodInterface $paymentMethod
     * @param bool $active
     */
    private function setPaymentMethodActive(Context $context, PaymentMethodInterface $paymentMethod, bool $active): void
    {
        $paymentMethodId = $this->getPaymentMethodId($paymentMethod, $context);

        if (!$paymentMethodId)
            return;

        $data = [
            'id'     => $paymentMethodId,
            'active' => $active,
        ];

        $this->paymentMethodRepository->upsert([$data], $context);
    }


    private function getPaymentMethods(): array
    {
        $aPaymentMethods = [
            AfterpayMethod::class,
            BancontactMethod::class,
            CreditcardMethod::class,
            IdealMethod::class,
            MaestroMethod::class,
            MastercardMethod::class,
            PaypalMethod::class,
            VisaMethod::class,
            VpayMethod::class,
        ];

        return $aPaymentMethods;
    }

    /**
     * @param Context $context
     * @param PaymentMethodInterface $paymentMethod
     * @param bool $active
     */
    private function installPaymentMethod(Context $context, PaymentMethodInterface $paymentMethod, bool $active): void {
        $paymentMethodId = $this->getPaymentMethodId($paymentMethod, $context);

        $pluginId = $this->pluginIdProvider->getPluginIdByBaseClass(Code1Rok2::class, $context);
        $mediaId = $this->getMediaId($paymentMethod, $context);

        $options = [
            'id'                => $paymentMethodId,
            'name'              => $paymentMethod->getName(),
            'description'       => $paymentMethod->getDescription(),
            'handlerIdentifier' => $paymentMethod->getPaymentHandler(),
            'position'          => $paymentMethod->getPosition(),
            'pluginId'          => $pluginId,
            'mediaId'           => $mediaId,
            'translations'      => $paymentMethod->getTranslations(),
        ];

        if ($paymentMethodId === null && $active) {
            $options['active'] = true;
        }

        $this->paymentMethodRepository->upsert([$options], $context);
    }


    /**
     * @param PaymentMethodInterface $paymentMethod
     * @param Context $context
     * @throws \Shopware\Core\Content\Media\Exception\DuplicatedMediaFileNameException
     * @throws \Shopware\Core\Content\Media\Exception\EmptyMediaFilenameException
     * @throws \Shopware\Core\Content\Media\Exception\IllegalFileNameException
     * @throws \Shopware\Core\Content\Media\Exception\MediaNotFoundException
     */
    private function addMedia(Context $context, PaymentMethodInterface $paymentMethod): void
    {
        $mediaFile = __DIR__  . '/../Resources/views/storefront/omnikassa/logo/'.$paymentMethod->getPaymentCode().'.png';

		if (!file_exists($mediaFile)) {
            return;
        }

        if ($this->hasMediaAlreadyInstalled($paymentMethod, $context)) {
            return;
        }

        $mediaFile = $this->createMediaFile($mediaFile);
        $mediaId = Uuid::randomHex();

        $this->mediaRepository->create(
            [
                [
                    'id' => $mediaId
                ]
            ],
            $context
        );

        $this->fileSaver->persistFileToMedia(
            $mediaFile,
            $this->getMediaName($paymentMethod),
            $mediaId,
            $context
        );
    }


    /**
     * @param string $filePath
     * @return MediaFile
     */
    private function createMediaFile(string $filePath): MediaFile
    {
        return new MediaFile(
            $filePath,
            mime_content_type($filePath),
            pathinfo($filePath, PATHINFO_EXTENSION),
            filesize($filePath)
        );
    }

    /**
     * @param PaymentMethodInterface $paymentMethod
     * @param Context $context
     * @return bool
     * @throws \Shopware\Core\Framework\DataAbstractionLayer\Exception\InconsistentCriteriaIdsException
     */
    private function hasMediaAlreadyInstalled(PaymentMethodInterface $paymentMethod, Context $context) : bool
    {
        $criteria = (new Criteria())->addFilter(
            new EqualsFilter(
                'fileName',
                $this->getMediaName($paymentMethod)
            )
        );

        /** @var MediaEntity $media */
        $media = $this->mediaRepository->search($criteria, $context)->first();

        return $media ? true : false;
    }

    /**
     * @param PaymentMethodInterface $paymentMethod
     * @param Context $context
     * @return string|null
     * @throws \Shopware\Core\Framework\DataAbstractionLayer\Exception\InconsistentCriteriaIdsException
     */
    private function getMediaId(PaymentMethodInterface $paymentMethod, Context $context): ?string
    {
        $criteria = (new Criteria())->addFilter(
            new EqualsFilter(
                'fileName',
                $this->getMediaName($paymentMethod)
            )
        );

        /** @var MediaEntity $media */
        $media = $this->mediaRepository->search($criteria, $context)->first();

        if (!$media) {
            return null;
        }

        return $media->getId();
    }

    /**
     * @param PaymentMethodInterface $paymentMethod
     * @return string
     */
    private function getMediaName(PaymentMethodInterface $paymentMethod): string
    {
        return 'omnikassa_' . $paymentMethod->getPaymentCode();
    }

    /**
     * @param PaymentMethodInterface $paymentMethod
     * @param Context $context
     * @return string|null
     */
    private function getPaymentMethodId(PaymentMethodInterface $paymentMethod, Context $context): ?string
    {
        // Fetch ID for update
        $paymentCriteria = (new Criteria())->addFilter(new EqualsFilter('handlerIdentifier', $paymentMethod->getPaymentHandler()));
        $paymentIds = $this->paymentMethodRepository->searchIds(
            $paymentCriteria,
            $context
        );

        if ($paymentIds->getTotal() === 0) {
            return null;
        }

        return $paymentIds->getIds()[0];
    }
}