<?php declare(strict_types=1);

namespace Omnikassa\Payment\PaymentHandlers;

use Exception;

use Shopware\Core\Checkout\Order\Aggregate\OrderTransaction\OrderTransactionStates;
use Shopware\Core\Checkout\Payment\Exception\AsyncPaymentFinalizeException;
use Shopware\Core\Framework\DataAbstractionLayer\EntityRepositoryInterface;
use Shopware\Core\Framework\Plugin\PluginService;

use Omnikassa\Payment\Helpers\OmnikassaHelper;
use Omnikassa\Payment\Helpers\RedirectHelper;

use Shopware\Core\Checkout\Order\Aggregate\OrderTransaction\OrderTransactionStateHandler;
use Shopware\Core\Checkout\Payment\Cart\AsyncPaymentTransactionStruct;
use Shopware\Core\Checkout\Payment\Cart\PaymentHandler\AsynchronousPaymentHandlerInterface;
use Shopware\Core\Checkout\Payment\Exception\AsyncPaymentProcessException;
use Shopware\Core\Checkout\Payment\Exception\CustomerCanceledAsyncPaymentException;
use Shopware\Core\Framework\Validation\DataBag\RequestDataBag;
use Shopware\Core\System\SalesChannel\SalesChannelContext;

use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpFoundation\Request;

use Shopware\Core\Framework\Language\LanguageEntity;
use Symfony\Contracts\Translation\TranslatorInterface;

use Psr\Log\LoggerInterface;

class AsyncOmnikassaHandler extends BaseOmnikassaHandler implements AsynchronousPaymentHandlerInterface
{

    /** @var RedirectHelper */
    protected $oRedirectHelper;

    /**
     * AsyncOmnikassaHandler constructor.
     * @param OrderTransactionStateHandler $oOrderTransactionStateHandler
     * @param OmnikassaHelper $omnikassaHelper
     * @param PluginService $oPluginService
     * @param RedirectHelper $oRedirectHelper
     */
    public function __construct(
        OrderTransactionStateHandler $oOrderTransactionStateHandler,
        OmnikassaHelper $omnikassaHelper,
        PluginService $oPluginService,
        RedirectHelper $oRedirectHelper,
        EntityRepositoryInterface $oOrderRepository,
        TranslatorInterface $oTranslator,
        LoggerInterface $oLogger
    )
    {
        $this->redirectHelper = $oRedirectHelper;
        parent::__construct($oOrderTransactionStateHandler,
            $omnikassaHelper,
            $oPluginService,
            $oOrderRepository,
            $oTranslator,
            $oLogger
        );
    }

    /**
     * @param AsyncPaymentTransactionStruct $oTransaction
     * @param RequestDataBag $oDataBag
     * @param SalesChannelContext $oSalesChannelContext
     * @return RedirectResponse
     * @throws AsyncPaymentProcessException
     */
    public function pay(
        AsyncPaymentTransactionStruct $oTransaction,
        RequestDataBag $oDataBag,
        SalesChannelContext $oSalesChannelContext
    ): RedirectResponse {
        // Method that sends the return URL to the external gateway and gets a redirect URL back

        $oOrder = $oTransaction->getOrder();
        // $sReturnUrl = $this->redirectHelper->encode($oTransaction->getReturnUrl());

        try {
            $sRedirectUrl = parent::doPayment(
				$oOrder,
				$oDataBag,
				$oSalesChannelContext,
				$oTransaction->getOrderTransaction(),
				$oTransaction->getReturnUrl()
			);

        } catch (Exception $e)
		{
            $this->oLogger->error(sprintf('Async Payment Exception, transaction: ' . $oTransaction->getOrderTransaction()->getId() . ', ' . $e->getMessage()));
            throw new AsyncPaymentProcessException($oTransaction->getOrderTransaction()->getId(),$e->getMessage());
        }

        // Redirect to external gateway
        return new RedirectResponse($sRedirectUrl);
    }

    /**
     * @throws CustomerCanceledAsyncPaymentException
     * @throws Exception
     */
    public function finalize(
        AsyncPaymentTransactionStruct $oTransaction,
        Request $oRequest,
        SalesChannelContext $oSalesChannelContext
    ): void {
		$sTransactionId = $oTransaction->getOrderTransaction()->getId();
        $sSalesChannelId = $oSalesChannelContext->getSalesChannel()->getId();

        $sOrderId = $oRequest->query->getAlnum('order_id');
        $sStatus = $oRequest->query->getAlpha('status');;
        $sSignature = $oRequest->query->getAlnum('signature');

		$bSandbox = $this->omnikassaHelper->getSetting('sandbox', $sSalesChannelId);

		if($bSandbox)
		{
			$sSigningKey = $this->omnikassaHelper->getSetting('testSigningkey', $sSalesChannelId);
		}
		else
		{
			$sSigningKey = $this->omnikassaHelper->getSetting('prodSigningkey', $sSalesChannelId);
		}

		$sHashString = $sOrderId . ',' . $sStatus;
		$sHash = hash_hmac('sha512', $sHashString, base64_decode($sSigningKey));

		if(strcmp($sSignature, $sHash) !== 0)
		{
			$this->oLogger->error(sprintf('Transaction %s, Invalid Return! Hash mismatch', $sTransactionId));
            throw new AsyncPaymentFinalizeException($sTransactionId, 'Invalid return!');
		}

		$oContext = $oSalesChannelContext->getContext();

        $sCurrentState = $oTransaction->getOrderTransaction()->getStateMachineState()->getTechnicalName();

        if(!in_array($sCurrentState , [
			OrderTransactionStates::STATE_OPEN,
			OrderTransactionStates::STATE_IN_PROGRESS,
			OrderTransactionStates::STATE_REMINDED]) ) {
            // Order already processed!
            return;
        }

		if(strcmp($sStatus, 'COMPLETED') === 0) {
			if (OrderTransactionStates::STATE_PAID == $sCurrentState)
			{
				// Order already processed!
				return;
			}

            $this->transactionStateHandler->paid($oTransaction->getOrderTransaction()->getId(), $oContext);
		}
		elseif(in_array($sOmniKassaStatus, array('CANCELLED', 'EXPIRED'))) {
			if(OrderTransactionStates::STATE_OPEN != $sCurrentState) {
				// Order already processed!
				return;
			}

            $this->oOrderTransactionStateHandler->cancel($oTransaction->getOrderTransaction()->getId(), $oContext);
		}
		elseif (strcmp($sStatus, 'IN_PROGRESS') === 0) {
			if(OrderTransactionStates::STATE_OPEN != $sCurrentState) {
				// Order already processed!
				return;
			}

			$this->transactionStateHandler->process($oTransaction->getOrderTransaction()->getId(), $oContext);
		}
		else {
			// INVALID STATE
			$this->oLogger->error(sprintf('Transaction %s, Unknown status found', $sTransactionId));
            throw new AsyncPaymentFinalizeException($sTransactionId, 'Unknown status found');
		}
    }
}