<?php declare(strict_types=1);

namespace Omnikassa\Payment\PaymentHandlers;

use Exception;

use Shopware\Core\Checkout\Order\Aggregate\OrderTransaction\OrderTransactionEntity;
use Shopware\Core\Checkout\Order\Aggregate\OrderTransaction\OrderTransactionStateHandler;
use Shopware\Core\Checkout\Order\Aggregate\OrderTransaction\OrderTransactionStates;
use Shopware\Core\Checkout\Order\OrderEntity;
use Shopware\Core\Checkout\Payment\Exception\UnknownPaymentMethodException;

use Shopware\Core\Framework\DataAbstractionLayer\EntityRepositoryInterface;
use Shopware\Core\Framework\Plugin\PluginService;
use Shopware\Core\Framework\Validation\DataBag\RequestDataBag;

use Shopware\Core\System\SalesChannel\SalesChannelContext;
use Shopware\Core\System\StateMachine\Exception\IllegalTransitionException;

use Omnikassa\Payment\Components\OmnikassaPayment\OmnikassaService;
use Omnikassa\Payment\Helpers\OmnikassaHelper;
use Omnikassa\Payment\PaymentMethods\PaymentMethodInterface;
use Omnikassa\Payment\Code1Rok2;

use Symfony\Contracts\Translation\TranslatorInterface;

use Psr\Log\LoggerInterface;

class BaseOmnikassaHandler {
    /**
     * @var PaymentMethodInterface
     */
    protected $oPaymentMethod;

    /**
     * @var OrderTransactionStateHandler
     */
    protected $oTransactionStateHandler;

    /**
     * @var OmnikassaHelper
     */
    protected $oOmnikassaHelper;

    /**
     * @var OmnikassaService;
     */
    protected $oOmnikassaService;

    /**
     * @var PluginService
     */
    protected $oPluginService;

    /** @var EntityRepositoryInterface */
    private $oOrderRepository;

    /** @var TranslatorInterface */
    private $oTranslator;

    /** @var LoggerInterface */
    protected $oLogger;


    /**
     * AsyncOmnikassaHandler constructor.
     * @param OrderTransactionStateHandler $oTransactionStateHandler
     * @param OmnikassaHelper $oOmnikassaHelper
     * @param PluginService $oPluginService
     * @param EntityRepositoryInterface $oOrderRepository
     */
    public function __construct(
        OrderTransactionStateHandler $oTransactionStateHandler,
        OmnikassaHelper $oOmnikassaHelper,
        PluginService $oPluginService,
        EntityRepositoryInterface $oOrderRepository,
        TranslatorInterface $oTranslator,
        LoggerInterface $oLogger
    )
    {
        $this->transactionStateHandler = $oTransactionStateHandler;
        $this->omnikassaHelper = $oOmnikassaHelper;
        $this->pluginService = $oPluginService;
        $this->orderRepository = $oOrderRepository;
        $this->translator = $oTranslator;
        $this->logger = $oLogger;
    }

    /**
     * @param OrderEntity $oOrder
     * @param RequestDataBag $oDataBag
     * @param SalesChannelContext $oSalesChannelContext
     * @param OrderTransactionEntity $oTransaction
     * @param string $sReturnUrl
     * @return string|null;
     * @throws Exception
     */
    public function doPayment(OrderEntity $oOrder,
							RequestDataBag $oDataBag,
							SalesChannelContext $oSalesChannelContext,
							OrderTransactionEntity $oTransaction,
							string $sReturnUrl = "") {

        $sSalesChannelId = $oOrder->getSalesChannelId() ?: $oSalesChannelContext->getSalesChannel()->getId();

		$oOmnikassa = $this->omnikassaHelper->initOmnikassaClient($sSalesChannelId);


		$oCustomer = $oSalesChannelContext->getCustomer();
		$sPaymentCode = $this->paymentMethod->getPaymentCode();

        $oPlugin = $this->pluginService->getPluginByName('Code1Rok2', $oSalesChannelContext->getContext());

        if (!$oPlugin) {
            throw new UnknownPaymentMethodException($oSalesChannelContext->getPaymentMethod()->getId());
        }

        if ($oPlugin->getBaseClass() !== Code1Rok2::class) {
            $this->logger->error('Not a Omnikassa payment method');
            throw new Exception(
                'Not a Omnikassa payment method' . PHP_EOL
            );
        }

		// Customer data is only required when using afterpay
        if (($oCustomer == null) && (strcmp($sPaymentCode, 'afterpay') === 0)) {
            $this->logger->error('No customer details');
            throw new Exception(
                'No customer details' . PHP_EOL
            );
        }

		$sCurrency = $oSalesChannelContext->getSalesChannel()->getCurrency()->getIsoCode();
        // $sLocale = $oSalesChannelContext->getSalesChannel()->getLanguage() !== null ?? $oSalesChannelContext->getSalesChannel()->getLanguage()->getLocale();

		// Set transaction parameters
        $oOmnikassa->setOrder($oOrder->getOrderNumber());
		$oOmnikassa->setPaymentMethod($sPaymentCode);
		// $oOmnikassa->setLanguageCode($sLocale);

		// Amount
		$oOmnikassa->setOrderAmount(round($oOrder->getAmountTotal() * 100));
		$oOmnikassa->setCurrencyCode($oSalesChannelContext->getCurrency()->getIsoCode());

		// Set return URL
		$oOmnikassa->setReturnUrl($sReturnUrl);

		$oOmnikassa->setAccessToken();

		if($sPaymentCode == 'afterpay')
		{
			$oPaymentAddress = $oCustomer->getDefaultBillingAddress();
            $oShipmentAddress = $oCustomer->getDefaultShippingAddress();

            // Collect billing address
			$aBillingAddress = array();

			$aAddress = $this->omnikassaHelper->mb_splitAddress($oPaymentAddress->getStreet());

			$aBillingAddress['firstName'] = substr($oPaymentAddress->getFirstName(), 0, 20);
			$aBillingAddress['middleName'] = '';
			$aBillingAddress['lastName'] = substr($oPaymentAddress->getLastName(), 0, 50);
			$aBillingAddress['street'] = substr($aAddress[0], 0, 100);
			$aBillingAddress['houseNumber'] = substr($aAddress[1], 0, 5);
			$aBillingAddress['houseNumberAddition'] = $aAddress[2];
			$aBillingAddress['postalCode'] = substr($oPaymentAddress->getZipCode(), 0, 10);
			$aBillingAddress['city'] = substr($oPaymentAddress->getCity(), 0, 86);
			$aBillingAddress['countryCode'] = substr($oPaymentAddress->getCountry()->getIso(), 0, 2);


			$aShippingAddress = array();

			$aAddress = $this->omnikassaHelper->mb_splitAddress($oShipmentAddress->getStreet());

			$aShippingAddress['firstName'] = substr($oShipmentAddress->getFirstName(), 0, 20);
			$aShippingAddress['middleName'] = '';
			$aShippingAddress['lastName'] = substr($oShipmentAddress->getLastName(), 0, 50);
			$aShippingAddress['street'] = substr($aAddress[0], 0, 100);
			$aShippingAddress['houseNumber'] = substr($aAddress[1], 0, 5);
			$aShippingAddress['houseNumberAddition'] = $aAddress[2];
			$aShippingAddress['postalCode'] = substr($oShipmentAddress->getZipCode(), 0, 10);
			$aShippingAddress['city'] = substr($oShipmentAddress->getCity(), 0, 86);
			$aShippingAddress['countryCode'] = substr($oShipmentAddress->getCountry()->getIso(), 0, 2);



            // Add Products
            $oCartProducts = $oOrder->getLineItems();
            $aOrderLines = array();

            foreach ($oCartProducts as $oProduct)
			{
                $aOrderLine = array();

                $aOrderLine['id'] = substr($oProduct->getId(), 0, 25);

                $sProductName = preg_replace('/[^A-Za-z0-9 -]/', '', $oProduct->getLabel());
                $aOrderLine['name'] = substr($sProductName, 0, 25);

                $sDescription = html_entity_decode($sProductName);
                $sDescription = strip_tags($sDescription);
                $sDescription = trim($sDescription);

                $aOrderLine['description'] = substr($sDescription, 0, 100);
                $aOrderLine['quantity'] = (int)($oProduct->getQuantity());

				$fUnitPrice = $oProduct->getUnitPrice();
                $fTaxRate = '0';

				if ($oProduct->getPrice()->getCalculatedTaxes() !== null && $oProduct->getPrice()->getCalculatedTaxes()->count() > 0) {
                    $fTaxRate = $oProduct->getPrice()->getCalculatedTaxes()->first()->getTaxRate();
                }

                $fUnitPriceExcl = $fUnitPrice;

                if ($fUnitPrice && $fTaxRate) {
                    $fUnitPriceExcl /= (1 + ($fTaxRate / 100));
                }

                $fUnitTax = $fUnitPrice - $fUnitPriceExcl;

                $fPriceIncl = round($fUnitPrice, 2);
                // $fPriceExcl = round($aProduct['price'], 2);
                $fPriceVat = round($fUnitTax, 2);
                $fVatPercentage = round($fTaxRate, 2);

                $aOrderLine['amount'] = array();
                $aOrderLine['amount']['currency'] = 'EUR';
                $aOrderLine['amount']['amount'] = (int)(round(($fPriceIncl), 2) * 100);

                $aOrderLine['tax'] = array();
                $aOrderLine['tax']['currency'] = 'EUR';
                $aOrderLine['tax']['amount'] = (int)(round($fPriceVat, 2) * 100);

                $iVatCategory = $this->checkVatCategory($fVatPercentage);

                $aOrderLine['vatCategory'] = $iVatCategory;

                $sProductType = 'PHYSICAL';
                $aOrderLine['category'] = $sProductType;

                $aOrderLines[] = $aOrderLine;
            }

            // Add Shipping Cost
            $oShipping = $oOrder->getShippingCosts();

            if ($oShipping !== null) {
                $aOrderLine = array();

                $fUnitPrice = $oShipping->getUnitPrice();
				$fTaxRate = '0';

				if  ($oShipping->getCalculatedTaxes() !== null && $oShipping->getCalculatedTaxes()->count() > 0) {
					$fTaxRate = $oShipping->getCalculatedTaxes()->first()->getTaxRate();
				}

				$fUnitPriceExcl = $fUnitPrice;

				if ($fUnitPrice && $fTaxRate) {
					$fUnitPriceExcl /= (1 + ($fTaxRate / 100));
				}

				$fUnitTax = $fUnitPrice - $fUnitPriceExcl;
                $fVatPercentage = round($fTaxRate, 2);

                $aOrderLine['id'] = 'shipping';
                $aOrderLine['name'] = 'Verzendkosten';
                $aOrderLine['description'] = 'Verzendkosten';
                $aOrderLine['quantity'] = $oShipping->getQuantity();

                $aOrderLine['amount'] = array();
                $aOrderLine['amount']['currency'] = 'EUR';
                $aOrderLine['amount']['amount'] = (int)(round($fUnitPrice, 2) * 100);

                $aOrderLine['tax'] = array();
                $aOrderLine['tax']['currency'] = 'EUR';
                $aOrderLine['tax']['amount'] = (int)(round($fTaxRate, 2) * 100);

                $iVatCategory = $this->checkVatCategory($fVatPercentage);

                $aOrderLine['vatCategory'] = $iVatCategory;
                $aOrderLine['category'] = 'PHYSICAL';

                $aOrderLines[] = $aOrderLine;
            }

			$sGender = $oDataBag->get('omnikassa_gender');
            $sBirthday = $oDataBag->get('omnikkassa_birthday');

            $aCustomerData = array();

            $aInitialParts = explode(' ', $aBillingAddress['firstName']);
            $sInitials = '';

            foreach ($aInitialParts as $sNamePart) {
                $sInitials .= substr($sNamePart, 0, 1);
            }

            $aCustomerData['gender'] = $sGender;
            $aCustomerData['telephoneNumber'] = $oPaymentAddress->getPhoneNumber();
            $aCustomerData['initials'] = $sInitials;
            $aCustomerData['emailAddress'] = $oCustomer->getEmail();

            $oOmnikassa->setCustomerBillingData($aBillingAddress);
            $oOmnikassa->setCustomerShippingData($aShippingAddress);
            $oOmnikassa->setAdditionalCustomerData($aCustomerData);
            $oOmnikassa->setProductData($aOrderLines);
        }

		if($oOmnikassa->getTransaction())
		{
			$aCustomFields = $oTransaction->getCustomFields() ?? [];
            $aCustomFields['omnikassa_transaction_id'] = $oOmnikassa->getTransactionId();

            $oTransaction->setCustomFields($aCustomFields);

            $aUpdate = [
                'id' => $oOrder->getId(),
                'customFields' => $aCustomFields,
            ];

            $this->orderRepository->update([$aUpdate], $oSalesChannelContext->getContext());

			return $sRedirectUrl = $oOmnikassa->getTransactionUrl();
		}
		else {
			// Error occured
			$sErrorMessage = 'Order ' . $oOrder->getOrderNumber() . ', Failed to start Transaction : ' . $oOmnikassa->getError();

			$this->logger->error($sErrorMessage);

			throw new Exception(
                $sErrorMessage
            );
		}
    }

	public function checkVatCategory($fVatPercentage)
	{
		$iVatCategory = 4;

		if($fVatPercentage == 21.00)
		{
			$iVatCategory = 1;
		}
		elseif($fVatPercentage == 9.00)
		{
			$iVatCategory = 2;
		}
		elseif($fVatPercentage == 0.00)
		{
			$iVatCategory = 3;
		}

		return $iVatCategory;
	}
}